@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2018  Kipp Cannon
+# Copyright (C) 2007--2021  Kipp Cannon
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the
@@ -38,6 +38,7 @@ import os
 import re
 import shutil
 import signal
+import sqlite3
 import sys
 import tempfile
 import threading
@@ -347,49 +348,23 @@ class workingcopy(object):
 		return False
 
 
-	def set_temp_store_directory(self, connection, verbose = False):
+	def set_temp_store_directory(self, connection):
 		"""
 		Sets the temp_store_directory parameter in sqlite.
 		"""
-		if verbose:
-			sys.stderr.write("setting the temp_store_directory to %s ... " % self.tmp_path)
+		if self.tmp_path is None:
+			if self.verbose:
+				sys.stderr.write("sqlite temp_store_directory not changed")
+			return
+		if self.verbose:
+			sys.stderr.write("setting the sqlite temp_store_directory to %s ... " % self.tmp_path)
 		cursor = connection.cursor()
 		cursor.execute("PRAGMA temp_store_directory = '%s'" % self.tmp_path)
 		cursor.close()
-		if verbose:
+		if self.verbose:
 			sys.stderr.write("done\n")
 
 
-#
-# backwards compatibility for old code.  FIXME:  delete in next release
-#
-
-
-def get_connection_filename(*args, **kwargs):
-	return workingcopy(*args, **kwargs).__enter__()
-
-
-def put_connection_filename(ignored, target, verbose = False):
-	target.verbose = verbose
-	target.__exit__(None, None, None)
-
-def discard_connection_filename(ignored, target, verbose = False):
-	target.discard = True
-	target.verbose = verbose
-	target.__exit__(None, None, None)
-
-def set_temp_store_directory(connection, temp_store_directory, verbose = False):
-	if temp_store_directory == "_CONDOR_SCRATCH_DIR":
-		temp_store_directory = os.getenv("_CONDOR_SCRATCH_DIR")
-	if verbose:
-		sys.stderr.write("setting the temp_store_directory to %s ... " % temp_store_directory)
-	cursor = connection.cursor()
-	cursor.execute("PRAGMA temp_store_directory = '%s'" % temp_store_directory)
-	cursor.close()
-	if verbose:
-		sys.stderr.write("done\n")
-
-
 #
 # =============================================================================
 #
@@ -399,66 +374,88 @@ def set_temp_store_directory(connection, temp_store_directory, verbose = False):
 #
 
 
-def idmap_create(connection):
+class idmapper(object):
 	"""
-	Create the _idmap_ table.  This table has columns "table_name",
-	"old", and "new" mapping old IDs to new IDs for each table.  The
-	(table_name, old) column pair is a primary key (is indexed and must
-	contain unique entries).  The table is created as a temporary
-	table, so it will be automatically dropped when the database
-	connection is closed.
+	Create and manage the _idmap_ table in an sqlite database.  This
+	table has columns "table_name", "old", and "new" mapping old IDs to
+	new IDs for each table.  The (table_name, old) column pair is a
+	primary key (is indexed and must contain unique entries).  The
+	table is created as a temporary table, so it will be automatically
+	dropped when the database connection is closed.
 
-	This function is for internal use, it forms part of the code used
-	to re-map row IDs when merging multiple documents.
+	This class is for internal use, it forms part of the code used to
+	re-map row IDs when merging multiple documents.
 	"""
-	connection.cursor().execute("CREATE TEMPORARY TABLE _idmap_ (table_name TEXT NOT NULL, old INTEGER NOT NULL, new INTEGER NOT NULL, PRIMARY KEY (table_name, old))")
+	def __init__(self, connection):
+		self.connection = connection
+		self.cursor = self.connection.cursor()
+		try:
+			self.cursor.execute("CREATE TEMPORARY TABLE _idmap_ (table_name TEXT NOT NULL, old INTEGER NOT NULL, new INTEGER NOT NULL, PRIMARY KEY (table_name, old))")
+		except sqlite3.OperationalError:
+			# assume table already exists
+			pass
+		self.sync()
 
+	def reset(self):
+		"""
+		Erase the contents of the _idmap_ table, but leave the
+		table in place.
+		"""
+		self.cursor.execute("DELETE FROM _idmap_")
 
-def idmap_reset(connection):
-	"""
-	Erase the contents of the _idmap_ table, but leave the table in
-	place.
+	def sync(self):
+		"""
+		Iterate over the tables in the database, ensure that there
+		exists a custom DBTable class for each, and synchronize
+		that table's ID generator to the ID values in the database.
+		"""
+		xmldoc = get_xml(self.connection)
+		for tbl in xmldoc.getElementsByTagName(DBTable.tagName):
+			tbl.sync_next_id()
+		xmldoc.unlink()
 
-	This function is for internal use, it forms part of the code used
-	to re-map row IDs when merging multiple documents.
-	"""
-	connection.cursor().execute("DELETE FROM _idmap_")
+	@staticmethod
+	def get_new(cursor, table_name, old, tbl):
+		"""
+		From the old ID, obtain a replacement ID by either grabbing
+		it from the _idmap_ table if one has already been assigned
+		to the old ID, or by using the current value of the Table
+		instance's next_id class attribute.  In the latter case,
+		the new ID is recorded in the _idmap_ table, and the class
+		attribute incremented by 1.
+		"""
+		cursor.execute("SELECT new FROM _idmap_ WHERE table_name == ? AND old == ?", (table_name, old))
+		new = cursor.fetchone()
+		if new is not None:
+			# a new ID has already been created for this old ID
+			return new[0]
+		# this ID was not found in _idmap_ table, assign a new ID and
+		# record it
+		new = tbl.get_next_id()
+		cursor.execute("INSERT INTO _idmap_ VALUES (?, ?, ?)", (table_name, old, new))
+		return new
 
-
-def idmap_sync(connection):
-	"""
-	Iterate over the tables in the database, ensure that there exists a
-	custom DBTable class for each, and synchronize that table's ID
-	generator to the ID values in the database.
-	"""
-	xmldoc = get_xml(connection)
-	for tbl in xmldoc.getElementsByTagName(DBTable.tagName):
-		tbl.sync_next_id()
-	xmldoc.unlink()
-
-
-def idmap_get_new(cursor, table_name, old, tbl):
-	"""
-	From the old ID string, obtain a replacement ID string by either
-	grabbing it from the _idmap_ table if one has already been assigned
-	to the old ID, or by using the current value of the Table
-	instance's next_id class attribute.  In the latter case, the new ID
-	is recorded in the _idmap_ table, and the class attribute
-	incremented by 1.
-
-	This function is for internal use, it forms part of the code used
-	to re-map row IDs when merging multiple documents.
-	"""
-	cursor.execute("SELECT new FROM _idmap_ WHERE table_name == ? AND old == ?", (table_name, old))
-	new = cursor.fetchone()
-	if new is not None:
-		# a new ID has already been created for this old ID
-		return new[0]
-	# this ID was not found in _idmap_ table, assign a new ID and
-	# record it
-	new = tbl.get_next_id()
-	cursor.execute("INSERT INTO _idmap_ VALUES (?, ?, ?)", (table_name, old, new))
-	return new
+	def update_ids(self, xmldoc, verbose = False):
+		# NOTE:  it's critical that the xmldoc object be retrieved
+		# *before* the rows whose IDs need to be updated are
+		# inserted (which, of course, is done before before this
+		# method is called).  the xml retrieval resets the "last
+		# max row ID" values inside the table objects, so if
+		# retrieval of the xmldoc is deferred until after the rows
+		# are inserted, nothing will get updated.  therefore, the
+		# xmldoc needs to be retrieved ahead of time and passed
+		# separately to this method, even though it seems this
+		# method could reconstruct the xmldoc itself from the
+		# connection.
+		table_elems = xmldoc.getElementsByTagName(ligolw.Table.tagName)
+		for i, tbl in enumerate(table_elems):
+			if verbose:
+				sys.stderr.write("updating IDs: %d%%\r" % (100.0 * i / len(table_elems)))
+			tbl.applyKeyMapping()
+		if verbose:
+			sys.stderr.write("updating IDs: 100%\n")
+		# reset for next document
+		self.reset()
 
 
 #
@@ -493,18 +490,6 @@ def get_table_names(connection):
 	return [name for (name,) in cursor]
 
 
-def get_column_info(connection, table_name):
-	"""
-	Return an in order list of (name, type) tuples describing the
-	columns in the given table.
-	"""
-	cursor = connection.cursor()
-	cursor.execute("SELECT sql FROM sqlite_master WHERE type == 'table' AND name == ?", (table_name,))
-	statement, = cursor.fetchone()
-	coldefs = re.match(_sql_create_table_pattern, statement).groupdict()["coldefs"]
-	return [(coldef.groupdict()["name"], coldef.groupdict()["type"]) for coldef in re.finditer(_sql_coldef_pattern, coldefs) if coldef.groupdict()["name"].upper() not in ("PRIMARY", "UNIQUE", "CHECK")]
-
-
 def get_xml(connection, table_names = None):
 	"""
 	Construct an XML document tree wrapping around the contents of the
@@ -530,7 +515,7 @@ def get_xml(connection, table_names = None):
 		if table_elem.validcolumns is not None:
 			for name in table_elem.validcolumns:
 				destrip[table.Column.ColumnName(name)] = name
-		for column_name, column_type in get_column_info(connection, table_elem.Name):
+		for column_name, column_type in table_elem.get_column_info():
 			if table_elem.validcolumns is not None:
 				try:
 					column_name = destrip[column_name]
@@ -703,6 +688,16 @@ class DBTable(table.Table):
 		self.append_statement = "INSERT INTO %s (%s) VALUES (%s)" % (self.Name, ",".join(self.dbcolumnnames), params)
 		self.append_attrgetter = operator.attrgetter(*self.dbcolumnnames)
 
+	def get_column_info(self):
+		"""
+		Return an in order list of (name, type) tuples describing
+		the columns in this table.
+		"""
+		self.cursor.execute("SELECT sql FROM sqlite_master WHERE type == 'table' AND name == ?", (self.Name,))
+		statement, = self.cursor.fetchone()
+		coldefs = re.match(_sql_create_table_pattern, statement).groupdict()["coldefs"]
+		return [(coldef.groupdict()["name"], coldef.groupdict()["type"]) for coldef in re.finditer(_sql_coldef_pattern, coldefs) if coldef.groupdict()["name"].upper() not in ("PRIMARY", "UNIQUE", "CHECK")]
+
 	def sync_next_id(self):
 		if self.next_id is not None:
 			maxid = self.cursor.execute("SELECT MAX(%s) FROM %s" % (self.next_id.column_name, self.Name)).fetchone()[0]
@@ -761,7 +756,7 @@ class DBTable(table.Table):
 		if self.next_id is not None:
 			# assign (and record) a new ID before inserting the
 			# row to avoid collisions with existing rows
-			setattr(row, self.next_id.column_name, idmap_get_new(self.cursor, self.Name, getattr(row, self.next_id.column_name), self))
+			setattr(row, self.next_id.column_name, idmapper.get_new(self.cursor, self.Name, getattr(row, self.next_id.column_name), self))
 		self._append(row)
 		if self.remap_first_rowid is None:
 			self.remap_first_rowid = self.maxrowid()
