@@ -1,4 +1,4 @@
-# Copyright (C) 2006--2009,2012--2019  Kipp Cannon
+# Copyright (C) 2006--2009,2012--2020  Kipp Cannon
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the
@@ -51,23 +51,16 @@ from . import types as ligolwtypes
 
 def get_param(xmldoc, name):
 	"""
-	Scan xmldoc for a param named name.  Raises ValueError if not
-	exactly 1 such param is found.
+	Deprecated.  Use Param.get_param(xmldoc, name)
 	"""
-	params = Param.getParamsByName(xmldoc, name)
-	if len(params) != 1:
-		raise ValueError("document must contain exactly one %s param" % Param.ParamName(name))
-	return params[0]
+	return Param.get_param(xmldoc, name)
 
 
 def get_pyvalue(xml, name):
 	"""
-	Convenience wrapper for get_param() that recovers an instance of a
-	Python builtin type from a Param element.
+	Deprecated.  Use Param.get_param(xml, name).value
 	"""
-	# Note:  the Param is automatically parsed into the correct Python
-	# type, so this function is mostly a no-op.
-	return get_param(xml, name).pcdata
+	return Param.get_param(xml, name).value
 
 
 #
@@ -147,6 +140,23 @@ class Param(ligolw.Param):
 				fileobj.write(xmlescape(ligolwtypes.FormatFunc[self.Type](self.pcdata).strip(u"\"") or u" "))
 		fileobj.write(self.end_tag(u"") + u"\n")
 
+	@property
+	def value(self):
+		"""
+		Synonym of .pcdata.  Makes calling code easier to
+		understand.  In the parent class .pcdata is text only.
+		Here it has been translated into a native Python type, but
+		it's not obvious in calling code that that is what has
+		happened so it can be unclear when reading calling codes if
+		one should be expecting a string or a native value.  Using
+		this synonym can clarify the meaning.
+		"""
+		return self.pcdata
+
+	@value.setter
+	def value(self, value):
+		self.pcdata = value
+
 	@classmethod
 	def build(cls, name, Type, value, start = None, scale = None, unit = None, dataunit = None, comment = None):
 		"""
@@ -181,14 +191,24 @@ class Param(ligolw.Param):
 		Examples:
 
 		>>> import sys
-		>>> Param.from_pyvalue(u"float", 3.0).write(sys.stdout)
-		<Param Name="float:param" Type="real_8">3</Param>
-		>>> Param.from_pyvalue(u"string", u"test").write(sys.stdout)
-		<Param Name="string:param" Type="lstring">test</Param>
-		>>> Param.from_pyvalue(u"shortstring", u"").write(sys.stdout)
-		<Param Name="shortstring:param" Type="lstring"> </Param>
-		>>> Param.from_pyvalue(u"none", None).write(sys.stdout)
-		<Param Name="none:param" Type="None"></Param>
+		>>> # float
+		>>> Param.from_pyvalue(u"example", 3.0).write(sys.stdout)
+		<Param Name="example:param" Type="real_8">3</Param>
+		>>> # string
+		>>> Param.from_pyvalue(u"example", u"test").write(sys.stdout)
+		<Param Name="example:param" Type="lstring">test</Param>
+		>>> # short string (non-empty data = not NULL)
+		>>> Param.from_pyvalue(u"example", u"").write(sys.stdout)
+		<Param Name="example:param" Type="lstring"> </Param>
+		>>> # None (empty data = NULL)
+		>>> Param.from_pyvalue(u"example", None).write(sys.stdout)
+		<Param Name="example:param" Type="None"></Param>
+
+		Note that any type of Param may be NULL-valued.  These
+		examples demonstrate the use of the automatic encoding
+		helper function, which translates None into a None-typed
+		Param because it doesn't know what else it might be, but,
+		for example, a float-typed Param may also be set to None.
 		"""
 		if value is not None:
 			return cls.build(name, ligolwtypes.FromPyType[type(value)], value, **kwargs)
@@ -198,10 +218,30 @@ class Param(ligolw.Param):
 	def getParamsByName(cls, elem, name):
 		"""
 		Return a list of params with name name under elem.
+
+		See also .get_param().
 		"""
 		name = cls.ParamName(name)
 		return elem.getElements(lambda e: (e.tagName == cls.tagName) and (e.Name == name))
 
+	@classmethod
+	def get_param(cls, xmldoc, name = None):
+		"""
+		Scan xmldoc for a param named name.  Raises ValueError if
+		not exactly 1 such param is found.  If name is None
+		(default), then the .paramName attribute of this class is
+		used.  The Param class does not provide a .paramName
+		attribute, but sub-classes may choose to.
+
+		See also .getParamsByName().
+		"""
+		if name is None:
+			name = cls.paramName
+		elems = Param.getParamsByName(xmldoc, name)
+		if len(elems) != 1:
+			raise ValueError("document must contain exactly one %s param" % cls.ParamName(name))
+		return elems[0]
+
 
 #
 # =============================================================================
