@@ -1,4 +1,4 @@
-# Copyright (C) 2006--2014  Kipp Cannon
+# Copyright (C) 2006--2020  Kipp Cannon
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the
@@ -29,10 +29,11 @@ Library of utility code for LIGO Light Weight XML applications.
 """
 
 
+import bz2
 import codecs
 import contextlib
 import gzip
-import warnings
+import lzma
 import os
 from six.moves import urllib
 import signal
@@ -309,22 +310,7 @@ class SignalsTrap(object):
 		return False
 
 
-# FIXME: remove the following once we drop the ``gz`` keyword argument.
-def _normalize_compress_kwarg(compress = None, gz = False):
-	if gz:
-		warnings.warn(
-			'The gz keyword argument is deprecated. '
-			'Use the compress keyword argument instead.',
-			DeprecationWarning,
-			stacklevel=3)
-		if compress is not None:
-			raise ValueError(
-				'must not specify both compress and gz keyword arguments')
-		compress = 'gz'
-	return compress
-
-
-def load_fileobj(fileobj, compress = None, gz = None, xmldoc = None, contenthandler = None):
+def load_fileobj(fileobj, compress = None, xmldoc = None, contenthandler = None):
 	"""
 	Parse the contents of the file object fileobj, and return the
 	contents as a LIGO Light Weight document tree.  The file object
@@ -333,9 +319,10 @@ def load_fileobj(fileobj, compress = None, gz = None, xmldoc = None, contenthand
 
 	The compress parameter selects the decompression algorithm to use.
 	Valid values are:  "auto" to automatically deduce the decompression
-	scheme from the file format, "gz" to force gzip decompression,
-	False to disable decompression, or None to select the default
-	behaviour (currently equivalent to "auto").
+	scheme from the file format;  one of "bz2", "gz", or "xz" to force
+	bzip2, gzip, or lzma/xz decompression, respectively;  False to
+	disable decompression;  or None to select the default behaviour
+	(which is "auto").
 
 	If the optional xmldoc argument is provided and not None, the
 	parsed XML tree will be appended to that document, otherwise a new
@@ -360,9 +347,6 @@ def load_fileobj(fileobj, compress = None, gz = None, xmldoc = None, contenthand
 	if contenthandler is None:
 		raise ValueError("missing required keyword argument \"contenthandler\"")
 
-	# FIXME: remove the following once we drop the ``gz`` keyword argument.
-	compress = _normalize_compress_kwarg(compress = compress, gz = gz)
-
 	if compress is None:
 		# select default behaviour
 		compress = "auto"
@@ -370,10 +354,20 @@ def load_fileobj(fileobj, compress = None, gz = None, xmldoc = None, contenthand
 	if compress == "auto":
 		# set keyword argument automatically from file format
 		fileobj = RewindableInputFile(fileobj)
-		magic = fileobj.read(2)
+		magic = fileobj.read(6)
 		fileobj.seek(0, os.SEEK_SET)
-		if magic == b"\037\213":
+		if magic[:3] == b"\x42\x5A\x68":
+			compress = "bz2"
+		elif magic[:2] == b"\x1F\x8B":
 			compress = "gz"
+		elif magic[:6] == b"\xFD\x37\x7A\x58\x5A\x00":
+			compress = "xz"
+		elif magic[:4] == b"\x28\xB5\x2F\xFD":
+			# NOTE:  only foramt detection is provided.  this
+			# compression format is not supported.  will
+			# trigger the unrecognized keyword arg exception
+			# below
+			compress = "zst"
 		else:
 			# format not recognized, assume not compressed
 			compress = False
@@ -385,9 +379,15 @@ def load_fileobj(fileobj, compress = None, gz = None, xmldoc = None, contenthand
 	if compress == False:
 		# pass-through
 		pass
+	elif compress == "bz2":
+		# bzip2 decompression
+		fileobj = bz2.BZ2File(fileobj, mode = "rb")
 	elif compress == "gz":
 		# gzip decompression
 		fileobj = gzip.GzipFile(mode = "rb", fileobj = fileobj if type(fileobj) == RewindableInputFile else RewindableInputFile(fileobj))
+	elif compress == "xz":
+		# xz/lzma decompression
+		fileobj = lzma.LZMAFile(fileobj, mode = "rb")
 	else:
 		# oops
 		raise ValueError("unrecognized compress \"%s\"" % compress)
@@ -435,8 +435,11 @@ def load_url(url, verbose = False, **kwargs):
 	as a LIGO Light Weight document tree.  Any source from which
 	Python's urllib library can read data is acceptable.  stdin is
 	parsed if url is None.  Helpful verbosity messages are printed to
-	stderr if verbose is True.  All other keyword arguments are passed
-	to load_fileobj(), see that function for more information.  In
+	stderr if verbose is True.  When url is not a local file,
+	urllib.request.urlopen() is used to access it, and if the keyword
+	arguments timeout and/or context is set then those arguments are
+	passed to urlopen().  All other keyword arguments are passed to
+	load_fileobj(), see that function for more information.  In
 	particular note that a content handler must be specified.
 
 	Example:
@@ -447,6 +450,7 @@ def load_url(url, verbose = False, **kwargs):
 	"""
 	if verbose:
 		sys.stderr.write("reading %s ...\n" % (("'%s'" % url) if url is not None else "stdin"))
+	urlopen_kwargs = dict((kwarg, kwargs.pop(kwarg)) for kwarg in ("context", "timeout") if kwarg in kwargs)
 	if url is None:
 		# In Python 3, ``sys.stdin`` has an attribute called
 		# ``buffer`` that is the underyling byte-oriented stream.
@@ -455,11 +459,11 @@ def load_url(url, verbose = False, **kwargs):
 	if scheme.lower() in ("", "file") and host.lower() in ("", "localhost"):
 		with open(path, "rb") as fileobj:
 			return load_fileobj(fileobj, **kwargs)
-	with contextlib.closing(urllib.request.urlopen(url)) as fileobj:
+	with contextlib.closing(urllib.request.urlopen(url, **urlopen_kwargs)) as fileobj:
 		return load_fileobj(fileobj, **kwargs)
 
 
-def write_fileobj(xmldoc, fileobj, compress = None, gz = False, compresslevel = 3, **kwargs):
+def write_fileobj(xmldoc, fileobj, compress = None, compresslevel = 3, **kwargs):
 	"""
 	Writes the LIGO Light Weight document tree rooted at xmldoc to the
 	given file object.  Internally, the .write() method of the xmldoc
@@ -468,10 +472,12 @@ def write_fileobj(xmldoc, fileobj, compress = None, gz = False, compresslevel =
 	object must be in binary mode.
 
 	The compress parameter selects the file compression format to use.
-	Valid values are:  False to disable compression, "gz" to select
-	gzip compression, None to select the default behaviour (disable
-	compression)..  When gzip compression is slected, the compresslevel
-	parameter sets the gzip compression level (the default is 3).
+	Valid values are:  False to disable compression;  one of "bz2",
+	"gz", or "xz" to select bzip2, gzip, or lzma compression,
+	respectively;  or None to select the default behaviour (which is to
+	disable compression).  When bzip2 or gzip compression is slected,
+	the compresslevel parameter sets the compression level (the default
+	is 3).
 
 	Example:
 
@@ -480,9 +486,6 @@ def write_fileobj(xmldoc, fileobj, compress = None, gz = False, compresslevel =
 	>>> xmldoc = load_filename("demo.xml", contenthandler = ligolw.LIGOLWContentHandler)
 	>>> write_fileobj(xmldoc, open("/dev/null","wb"))
 	"""
-	# FIXME: remove the following once we drop the ``gz`` keyword argument.
-	compress = _normalize_compress_kwarg(compress = compress, gz = gz)
-
 	if compress is None:
 		# select default behaviour
 		compress = False
@@ -495,8 +498,12 @@ def write_fileobj(xmldoc, fileobj, compress = None, gz = False, compresslevel =
 		if compress == False:
 			# no compression
 			pass
+		elif compress == "bz2":
+			fileobj = bz2.BZ2File(fileobj, mode = "wb", compresslevel = compresslevel)
 		elif compress == "gz":
 			fileobj = gzip.GzipFile(mode = "wb", fileobj = fileobj, compresslevel = compresslevel)
+		elif compress == "xz":
+			fileobj = lzma.LZMAFile(fileobj, mode = "wb", format = lzma.FORMAT_XZ)
 		else:
 			# oops
 			raise ValueError("unrecognized compress \"%s\"" % compress)
@@ -539,7 +546,7 @@ class tildefile(object):
 		return False
 
 
-def write_filename(xmldoc, filename, verbose = False, compress = None, gz = False, with_mv = True, trap_signals = SignalsTrap.default_signals, **kwargs):
+def write_filename(xmldoc, filename, verbose = False, compress = None, with_mv = True, trap_signals = SignalsTrap.default_signals, **kwargs):
 	"""
 	Writes the LIGO Light Weight document tree rooted at xmldoc to the
 	file name filename.  If filename is None the file is written to
@@ -549,10 +556,12 @@ def write_filename(xmldoc, filename, verbose = False, compress = None, gz = Fals
 
 	The compress keyword argument selects the compression format to
 	use.  Recognized values are:  "auto" to automatically select the
-	format based on filename, "gz" to force gzip compression, False to
-	disable compression, None to select the default behaviour (which is
-	currently "auto").  When "auto" is the mode selected, if the
-	filename does not match a recognized pattern or filename is None
+	format based on filename;  None to select the default behaviour
+	(which is "auto");  or any of the compression format values
+	recognized by write_fileobj().  When "auto" is the mode selected,
+	then if the filename ends in ".bz2", ".gz", or ".xz" then the
+	corresponding compression format is selected, othewrise if the
+	filename does not match a recognized pattern or if filename is None
 	(writing to stdout) then compression is disabled.
 
 	If with_mv is True and filename is not None the filename has a "~"
@@ -576,12 +585,13 @@ def write_filename(xmldoc, filename, verbose = False, compress = None, gz = Fals
 
 	Example:
 
+	>>> # write file
 	>>> write_filename(xmldoc, "demo.xml")	# doctest: +SKIP
-	>>> write_filename(xmldoc, "demo.xml.gz", compress = 'gz')	# doctest: +SKIP
+	>>> # write gzip-compressed file (auto-select format)
+	>>> write_filename(xmldoc, "demo.xml.gz")	# doctest: +SKIP
+	>>> # force compression off
+	>>> write_filename(xmldoc, "demo.xml.gz", compress = False)	# doctest: +SKIP
 	"""
-	# FIXME: remove the following once we drop the ``gz`` keyword argument.
-	compress = _normalize_compress_kwarg(compress = compress, gz = gz)
-
 	#
 	# select format
 	#
@@ -594,8 +604,17 @@ def write_filename(xmldoc, filename, verbose = False, compress = None, gz = Fals
 			# writing to stdout:  no filename = cannot deduce
 			# compression format = turn compression off
 			compress = False
+		elif filename.endswith(".bz2"):
+			compress = "bz2"
 		elif filename.endswith(".gz"):
 			compress = "gz"
+		elif filename.endswith(".xz"):
+			compress = "xz"
+		elif filename.endswith(".zst"):
+			# NOTE:  this format is not supported.  this will
+			# trigger the unrecognized keyword arg exception in
+			# write_fileobj()
+			compress = "zst"
 		else:
 			# filename scheme not recognized, disable
 			# compression
